;                       _______  _______
;                      |       \/       |
;                RA2 --+ 1           18 +-- RA1
;                      |                |     
;                RA3 --+ 2           17 +-- RA0
;                      |                |
;               RTCC --+ 3         >>16 +-- OSC1/CLKIN
;                      |                |
;              !MCLR --+ 4 <<        15 +-- OSC2/CLKOUT
;                      |                |
;                Vss --+ 5 <<      >>14 +-- Vdd
;                      |                |
;                RB0 --+ 6         >>13 +-- RB7  DATA IN/OUT
;                      |                |
;                RB1 --+ 7         >>12 +-- RB6 
;                      |                |
;                RB2 --+ 8         >>11 +-- RB5  SCL
;                      |                |
;                RB3 --+ 9         >>10 +-- RB4  SDA
;                      |                |
;                      +----------------+
;
;******************************************************

KENWOOD	EQU	0

SDA	EQU	4
SCL	EQU	5
I_O	EQU	7
TEST	EQU	0
;******************************************************

            LIST      P=16F84, F=INHX8M
		include "P16FXX.inc"

	IF KENWOOD
TIM_PAR  EQU	01BH
	ELSE
TIM_PAR  EQU	01AH
	ENDIF
;******************************************************

            ORG     0x0000

            CALL    Std_pause           ; poczekaj. powrot z W=0
            TRIS    PORTA               ;\ Define PORT A I/O
            ADDLW   0x7F
            OPTION                      ; Load OPTION register => 0

            MOVLW   0x0E                ; dlugosc ATR
            MOVWF   0x10                ; licznik
ATR_loop    MOVF    0x10,W              ;\
            CALL    Rd_eeprom           ;
            CALL    Send                ;   ATR
            DECFSZ  0x10,f              ;
            GOTO    ATR_loop            ;/

TX9000      MOVLW   0x90                ;\
            CALL    Send
TX00_0D     MOVLW   0x00                ; 9000
            BTFSC   PCLATH,03           ;
            MOVLW   0x1D                ; 901D    
            CALL    Send
            CALL    Receive             ; gubi komende C1 a powinien sprawdzic
                                        ; DOPISAC!
            MOVLW   0x0C                ;\ ustaw wskaznik bufora
            MOVWF   FSR                 ;/

Com_loop    CALL    Receive             ;\
            MOVWF   INDF
            INCF    FSR,f               ; pobierz 4 bytes komendy
            BTFSS   FSR,04              ; 0CH, 0DH, 0EH, 0FH
            GOTO    Com_loop            ;/ 

            MOVLW   0x0D                ; ilosc funkcji do przeskanowania
            MOVWF   0x10                ; licznik
Comm_scan   MOVF    0x10,W
            ADDLW   0x0E                ; offset w internal EEPROM  
            CALL    Rd_eeprom
            XORWF   0x0C,W
            BTFSC   STATUS,Z            ; Skip if not zero
            GOTO    Send_ack            ; komenda znaleziona
            DECFSZ  0x10,f              ;\
            GOTO    Comm_scan           ; szukaj dalej
            MOVLW   0x6D                ;/
            CALL    Send
            GOTO    TX00_0D

Send_ack    MOVF    0x0C,W              ;\ ACK
            CALL    Send                ;/
            MOVF    0x10,W
            ADDLW   0x2B                ;\
            CLRF    PCLATH
            MOVWF   PCL                 ;/ Absolute addressing
            GOTO    Instr_0E
            GOTO    Instr_16
            GOTO    Instr_12
            GOTO    Instr_34
            GOTO    Instr_32
            GOTO    Instr_20
            GOTO    Instr_30
            GOTO    Instr_21
            GOTO    Instr_3A
            GOTO    Instr_3C
            GOTO    Instr_40
            GOTO    Instr_1A

Instr_5A    BSF     PCLATH,03

;*************************************************
; function : 0E C1 CLASS instruction             *
; request smartcard serial number                *
; C1 0E 00 00 08                                 *
; 0E 00 00 00 00 00 12 34 56 90 00               *
; Serial number: 001.193.046                     *
;*************************************************
Instr_0E    MOVLW   0x00            ; adres w EEPROM
COMM_EEPR   CALL    EEPROM_send
            GOTO    TX9000


Instr_1A    INCF    PCLATH,f
            MOVLW   0xA6
            GOTO    COMM_EEPR

;*************************************************
; function : 16 C1 CLASS instruction             *
; ENTITIES SUPPORTED
;*************************************************
Instr_16    MOVLW   0x08            ; adres w EEPROM
            GOTO    COMM_EEPR

;****************************************************************************************
; function : 12 C1 CLASS instruction                                                    *
; this instruction requests an ID CAM                                                   *
; C1 12 aa 00 18                                                                        *
; aa is the ID provider                                                                 *
; 12 ii ii bb bb bb bb bb bb bb bb bb bb bb bb bb bb bb bb 00 xx xx yy zz zz 90 00      *
; ii ii  : provider channel                                                             *
; bb..bb : provider name                                                                *
; xx xx  : smartcard address                                                            *
; yy     : ??                                                                           *
; zz zz  : termination date  ( 7 bits year since 1990 + 4 bits month + 5 bits day       *
; data received 17H 18H 19H 1AH                                                         *
;                                                                                       *
;****************************************************************************************
Instr_12    MOVLW   0xF8           ; offset w EEPROM
            INCF    0x0D,f         ; zwieksz wskaznik
instr_12_loop  
            ADDLW   0x18           ; 
            BTFSC   STATUS,Z       ; Skip if not zero
            INCF    PCLATH,f
            DECFSZ  0x0D,f
            GOTO    instr_12_loop
            GOTO    COMM_EEPR

;*************************************************
; function : 3A C1 CLASS instruction             *
; request for the control word                   *
;C1 
;3A 00 00 10 
;3A 					; ACK
;10 10 55 E6 55 54 05 80 
;04 14 40 8F 40 41 11 23 
;90 00 
;*************************************************
Instr_3A    MOVLW   0x30
            MOVWF   FSR               ; wskaznik bufora w RAM
instr_3A_loop  
            MOVF    INDF,W            ; wez z bufora
            CALL    Send              ; wyslij
            INCF    FSR,f             ; zwieksz wskaznik
            DECFSZ  0x0F,f            ; odlicz dlugosc
            GOTO    instr_3A_loop
            GOTO    TX9000            ; zakoncz

;*************************************************
; function : 3C C1 CLASS instruction             *
; ENTITLEMENT CONTROL                            *
;C1 
;3C 						; komenda
;04 						; 4-ty provider na karcie,
;0C 						; klucz 0C
;27		 				; 27H bytes long
;3C						; ACK
;71 00 						; unknown
;00 10 						; provider
;07 D0 						; year
;00 08						; month
;27 						; separator
;15 0C 						; complete data
;13 21 						; boundle
;[D1]						; marker
;EF 44 F8 57 15 A0 A0 A6			; control word 1
;D0 11 1C 81 87 6E 24 8E			; control word 2
;82 						; separator
;23 C0 DF FB 10 19 62 DB 			; sygnatura
;90 00 
;*************************************************
Instr_3C    CALL    Get_CW         ; tu mozna ew. dopisac gaszenie LED'a
				           ; ID providera jest w 14h
            MOVF    0x14,W
            SUBLW   0x1D
            MOVWF   EEADR
            MOVLW   0x0C
            CALL    xor_eedat
            CALL    xor_2C_38
            MOVF    0x14,W         ;\   
            SUBLW   0x25
            MOVWF   EEADR
            MOVLW   0x0C           ;/
            CALL    xor_eedat
            CALL    xor_24_30
            CALL    Encrypt
            CALL    xor_F8_30
            CALL    Encrypt
            CALL    xor_F8_30
            GOTO    TX9000
;----------------------------------------------------
Get_CW      MOVF    0x0F,W        ; pobierz dlugosc
            MOVWF   0x10          ; zapamieaj licznik
GET_CW_loop MOVLW   0x31
            SUBWF   0x10,W
            BTFSS   STATUS,C      ; Skip if carry set
            GOTO    c068c
            MOVLW   0x08          ;
            SUBWF   0x10,f	    ; odejmij 8
            MOVWF   0x0F
            CALL    Doczytaj
            GOTO    GET_CW_loop

c068c       MOVLW   0x18          ;\ ustaw bufor
            MOVWF   FSR           ;/
c068c_loop  CALL    Receive       ;\
            MOVWF   INDF
            INCF    FSR,f
            DECFSZ  0x10,f
            GOTO    c068c_loop    ;/

            MOVF    FSR,W
            MOVWF   0x14          ; ID providera
            CALL    c112c         ; wylicz adres klucza w EEPROM
            MOVLW   0x40
            MOVWF   FSR
            CALL    Get_key       ; pobierz klucz z EEPROM
            MOVLW   0x40
            BTFSC   0x0D,03
            ADDWF   0x10,f
            CALL    Get_key       ; pobierz klucz z EEPROM
            BTFSS   PCLATH,04
            GOTO    c08Dc
            MOVLW   0x18
            CALL    xor_xx_30
            CALL    Encrypt
            MOVLW   0x18
            CALL    xor_xx_30
            MOVLW   0x10
            CALL    xor_xx_30
            CALL    Encrypt
            MOVLW   0x10
            CALL    xor_xx_30
            MOVLW   0x08
            CALL    xor_xx_30
            CALL    Encrypt
            MOVLW   0x08
            CALL    xor_xx_30
            CALL    c08Bc
c08Bc       CALL    xor_F8_30
            CALL    Encrypt
c08Dc       RETURN

xor_F8_30   MOVLW   0xF8
xor_xx_30   MOVWF   EEADR
            MOVLW   0x30
            CALL    xor_eedat
            RETURN

Encrypt     CLRF    0x0F
            MOVLW   0x02
            MOVWF   PCLATH
            CLRF    0x0D
Encrypt_loop  
            CALL    c0FFc
            INCF    0x0F,f
            MOVF    0x0F,W
            ANDLW   0x03
            BTFSC   STATUS,Z              ; Skip if not zero
            INCF    0x0D,f
            BTFSS   0x0F,06
            GOTO    Encrypt_loop
            MOVLW   0x0F
            MOVWF   0x0D
            MOVLW   0x0C
            MOVWF   0x0F
c0A3c       CLRW
            CALL    c10Cc
            XORWF   0x34,W
            CALL    Tablica
            MOVWF   0x10
            MOVLW   0x01
            CALL    c10Cc
            XORWF   0x35,W
            CALL    Tablica
            MOVWF   0x11
            MOVLW   0x02
            CALL    c10Cc
            XORWF   0x36,W
            CALL    Tablica
            MOVWF   0x12
            MOVLW   0x03
            CALL    c10Cc
            XORWF   0x37,W
            CALL    Tablica
            MOVWF   0x13
            INCF    PCLATH,f
            XORWF   0x10,f
            SWAPF   0x13,W
            ADDWF   0x12,W
            CALL    c1ADc
            MOVWF   0x12
            XORWF   0x13,f
            SWAPF   0x12,W
            ADDWF   0x11,W
            CALL    c1ADc
            MOVWF   0x11
            XORWF   0x12,f
            SWAPF   0x11,W
            ADDWF   0x10,W
            CALL    c1ADc
            MOVWF   0x10
            DECF    PCLATH,f
            XORWF   0x11,f
            SWAPF   0x10,W
            ADDWF   0x13,W
            CALL    Tablica
            MOVWF   0x13
            XORWF   0x10,f
            SWAPF   0x13,W
            ADDWF   0x12,W
            CALL    Tablica
            MOVWF   0x12
            XORWF   0x13,f
            SWAPF   0x12,W
            ADDWF   0x11,W
            CALL    Tablica
            MOVWF   0x11
            XORWF   0x12,f
            MOVF    0x10,W
            XORWF   0x11,f
            INCF    PCLATH,f
            MOVF    0x11,W
            CALL    c1ADc
            XORWF   0x30,f
            MOVF    0x13,W
            CALL    c1ADc
            XORWF   0x31,f
            MOVF    0x10,W
            CALL    c1ADc
            XORWF   0x32,f
            MOVF    0x12,W
            CALL    c1ADc
            XORWF   0x33,f
            DECF    PCLATH,f
            MOVLW   0x03
            ADDWF   0x0F,f
            CALL    c0FFc
            DECF    0x0F,f
            CALL    c0FFc
            DECF    0x0F,f
            CALL    c0FFc
            DECF    0x0F,f
            CALL    c0FFc
            MOVF    0x0D,f
            BTFSC   STATUS,Z
            RETURN

            MOVLW   0x04
            MOVWF   EEADR
            MOVLW   0x34
            CALL    c199c
            DECF    0x0D,f
            MOVLW   0xFC
            ADDWF   0x0F,f
            MOVLW   0x0C
            BTFSC   0x0F,07
            MOVWF   0x0F
            GOTO    c0A3c

c0FFc       MOVLW   0x01
            CALL    c10Cc
            MOVWF   0x0C
            MOVLW   0xFF
            CALL    c10Cc
            XORWF   0x0C,f
            CLRW
            CALL    c10Cc
            MOVF    0x0C,W
            XORWF   0x0D,W
            CALL    Tablica
            XORWF   INDF,f
            RETURN

c10Cc       ADDWF   0x0F,W
            ANDLW   0x0F
            MOVWF   FSR
            BSF     FSR, 06
            MOVF    INDF,W
            RETURN
;---------------
; wylicz adres klucza
c112c       CLRF    0x10
            RRF     0x0D,f
            RRF     0x10,f
            RRF     0x0D,W
            RRF     0x10,f
            ANDLW   0x03
            MOVWF   PCLATH
            BSF     PCLATH,02
            BTFSC   0x0E,07
            BSF     PCLATH,04
            MOVLW   0x07
            ANDWF   0x0E,f
c11Ec       MOVLW   0x08
            DECF    0x0E,f
            BTFSC   0x0E,07
            RETURN

            ADDWF   0x10,f
            GOTO    c11Ec
;-------------------------------------
; pobierz klucz z EEPROM
Get_key     MOVF    0x10,W
c125c       CALL i2c_wr                ; adres
            MOVLW   0x08               ; ilosc
            MOVWF   0x11               ; licznik
c125c_loop  CALL    i2c_rd
            MOVWF   INDF
            INCF    FSR,f
            DECFSZ  0x11,f
            GOTO    c125c_loop
            RETURN

;*************************************************
; function : 34 C1 CLASS instruction             *
; request for the control word                   *
;*************************************************
Instr_34    CALL    Doczytaj
            GOTO    TX9000

;*************************************************
; function : 32 C1 CLASS instruction             *
; extra and general infos                        *
;*************************************************
Instr_32    INCF    PCLATH,f
            MOVLW   0x99
            BTFSS   0x0F,00
            GOTO    c3A6c
            GOTO    COMM_EEPR

;*************************************************
; function : 20 C1 CLASS instruction             *
; pobranie danych do bufora i zapis do EEPROM    *
;*************************************************
Instr_20    MOVF    0x0F,W          ; pobierz dlugosc instrukcji
                                    ; zapamietaj
            MOVLW   0x38            ;\ ustaw wskaznik bufora
            MOVWF   FSR             ;/ 
instr_20_loop  
            CALL    Receive         ;\
            MOVWF   INDF            ; pobierz do bufora
            INCF    FSR,f
            BTFSS   FSR,06
            GOTO    instr_20_loop   ;/
            MOVF    0x0D,W          ; P1= adres
            ADDWF   PCLATH,f
            MOVF    0x0E,W          ; P2= adres
            CALL    i2c_wr          ; adres
            CALL    wr_i2c_string   ; zapisz dane do EEPROM'u
            GOTO    TX9000

;---------------------------------------------------
wr_i2c_string  
            MOVLW   0x37            ;\
            MOVWF   FSR             ;/ ustaw bufor
wr_i2c_str_loop  
            INCF    FSR,f
            MOVF    INDF,W
            CALL    wystaw_byte
            DECFSZ  0x0F,f
            GOTO    wr_i2c_str_loop
            GOTO    i2c_stop
;---------------------------------------------------
c14Dc       SUBWF   0x0F,f
Doczytaj    CALL    Receive        ;\ 
            DECFSZ  0x0F,f
            GOTO    Doczytaj       ;/
            RETURN

;========================================
; procedura wyslania bytes na I/O karty. Dziala ale ma KNOT!
; b. ladnie rowiazane liczenie sumy kontrolnej

Send        MOVWF   0x16
            MOVLW   0x32
            CALL    Pause
            MOVLW   0x7F           ;\
            TRIS    PORTB          ;/ Define PORT B I/O RB7=output
            CLRF    0x17
            BCF     PORTB,07       ; wystaw bit startu
            MOVLW   0x08           ;\ 
            MOVWF   0x14           ;/ licznik ilosci bitow
send_loop   CALL    Std_pause      ; czas jednego bitu
            MOVF    0x16,W         ; pobierz dana
            XORWF   0x17,f         ; suma kontrolna
            BCF     PORTB,07
            BTFSC   0x16,00        ; skok jesli bit 16H.0 = 0
            BSF     PORTB,07
            RRF     0x16,f         ; przesun 
            DECFSZ  0x14,f
            GOTO    send_loop      ;/
            CALL    Std_pause      ;\ 
            BCF     PORTB,07       ; ustaw 0    
            BTFSC   0x17,00        ; skok jesli bit sumy kontrolnej = 0
            BSF     PORTB,07       ; ustaw 1
            CALL    Std_pause      ;/
            MOVLW   0xFF           ;\ Define PORT B I/O = input
            TRIS    PORTB          ;/
            MOVLW   0x4F           ;\ parametr czasu
            GOTO    Pause          ;/ 2 bity stopu

;-----------------------------------------------
Receive     BTFSC   PORTB,07       ; skip if I/O = 0
            GOTO    Receive        ; czekaj na bit startu
            MOVLW   0x2A           ; parametr czasu
            CALL    Pause          ; przeczekaj bit startu
            MOVLW   0x09           ; ilosc bitow
            MOVWF   0x14           ; licznik
Receive_loop  
            BCF     STATUS,C       ; Clear carry flag
            BTFSC   PORTB,07
            BSF     STATUS,C       ; Set carry flag
            RRF     0x16,f         ; poobracaj...
            CALL    Std_pause      ;
            DECFSZ  0x14,f
            GOTO    Receive_loop   ; petla
            RLF     0x16,f         ; ustal dane
            MOVLW   0x28           ;\ parametr czasu
            CALL    Pause          ;/ olej bit sumy i bity stopu
            MOVF    0x16,W         ; odebrany byte do W
            RETURN
;-------------------------------------------------
I2C_busy    CALL    i2c_start
            BCF     STATUS,C       ; Clear carry flag
            CALL    wystaw_adres
            BTFSC   STATUS,C       ; Skip if no busy
            GOTO    I2C_busy
            RETURN
;-------------------------------------------------
Std_pause   MOVLW   TIM_PAR
Pause       MOVWF   0x15
Pause_loop  DECFSZ  0x15,f
            GOTO    Pause_loop
            RETLW   0x00
;-------------------------------------------------
Rd_eeprom   MOVWF   EEADR
            BSF     STATUS,RP0    ; Select page 1 (80H - FFH)
            BSF     EECON1,00
            BCF     STATUS,RP0    ; Select page 0 (00H - 7FH)
            MOVF    EEDATA,W
            RETURN
;---------------------------------------------------
EEPROM_send CALL    i2c_wr        ; adres
eepr_send_loop  
            DECF    0x0F,f        ; odlicz
            BTFSC   0x0F,07
            RETURN

            CALL    i2c_rd        ; czytaj
            CALL    Send          ; wyslij
            GOTO    eepr_send_loop
;--------------------------------------------------
xor_eedat   CALL    c199c
            GOTO    c19Ac
c199c       MOVWF   FSR
c19Ac       CALL    c19Dc
            CALL    c19Dc
            CALL    c19Dc
c19Dc       MOVF    INDF,W
            MOVWF   EEDATA
            MOVF    EEADR,W
            SUBWF   FSR,f
            MOVF    EEDATA,W
            XORWF   INDF,f
            MOVF    INDF,W
            XORWF   EEDATA,f
            MOVF    EEDATA,W
            XORWF   INDF,f
            MOVF    EEADR,W
            ADDWF   FSR,f
            MOVF    EEDATA,W
            MOVWF   INDF
            INCF    FSR,f
            RETURN
;-----------------------------------------------------
c1ADc       MOVWF   EEADR
            CALL    i2c_start
            BCF     STATUS,C               ; Clear carry flag
            CALL    wystaw_adres
            MOVF    EEADR,W
            CALL    wystaw_byte
i2c_rd      CALL    i2c_start
            BSF     STATUS,C               ; Set carry flag
            CALL    wystaw_adres
            CALL    wez_byte
            CALL    i2c_stop
            MOVF    EEDATA,W
            RETURN

i2c_wr      MOVWF   EEADR
            CALL    I2C_busy
            MOVF    EEADR,W
            GOTO    wystaw_byte

wr_last     CALL    wystaw_byte
            GOTO    i2c_stop
;-----------------------------------------
wystaw_adres  
            RLF     PCLATH,W
            ANDLW   0x0F                 ; wytnij mlodsza czesc adresowa
            IORLW   0xA0                 ; dodaj adres I2C
wystaw_byte MOVWF   EEDATA
            MOVLW   0xCF                 ;\
            TRIS    PORTB                ;/ Define PORT B I/O
            CALL    wystaw_nible
            CALL    wystaw_nible
            GOTO    wez_bit
wystaw_nible  
            CALL    wystaw_nible_2
            CALL    wystaw_nible_2
            CALL    wystaw_nible_2
wystaw_nible_2  
            RLF     EEDATA,f
            GOTO    wystaw_bit_x

wez_byte    MOVLW   0xDF                 ;\
            TRIS    PORTB                ;/ Define PORT B I/O
            CALL    wez_nible
            CALL    wez_nible
            BSF     STATUS,C             ; Set carry flag
            GOTO    wystaw_bit
wez_nible   CALL    wez_nible_2
            CALL    wez_nible_2
            CALL    wez_nible_2
wez_nible_2 CALL    wez_bit_x
            RLF     EEDATA,f
            RETURN

;-------------------------------------------------
i2c_start   BSF     PORTB,04
            MOVLW   0xCF                ;\
            TRIS    PORTB               ;/ Define PORT B I/O
            BSF     PORTB,05
            NOP
            BCF     PORTB,04
            GOTO    c1E9c
;-----------------------------------------------------
i2c_stop    BCF     PORTB,04
            MOVLW   0xCF
            TRIS    PORTB               ; Define PORT B I/O
            NOP
            BSF     PORTB,05
            NOP
            BSF     PORTB,04
c1E8c       NOP
c1E9c       BCF     PORTB,05
            RETURN

wystaw_bit  MOVLW   0xCF                ;\
            TRIS    PORTB               ;/ Define PORT B I/O
wystaw_bit_x  
            BTFSS   STATUS,C            ; Skip if carry set
            BCF     PORTB,04
            BTFSC   STATUS,C            ; Skip if no carry
            BSF     PORTB,04
            BSF     PORTB,05
            GOTO    c1E8c
;----------------------------------------------------
wez_bit     MOVLW   0xDF                ;\
            TRIS    PORTB               ;/ Define PORT B I/O
wez_bit_x   BSF     PORTB,05
            NOP
            BSF     STATUS,C            ; Set carry flag
            BTFSS   PORTB,04
            BCF     STATUS,C            ; Clear carry flag
            GOTO    c1E9c

Tablica     MOVWF   PCL    ; Absolute addressing
;---------------------------------------------
            ORG	 0200H

            RETLW   0x2A
            RETLW   0xE1
            RETLW   0x0B
            RETLW   0x13
            RETLW   0x3E
            RETLW   0x6E
            RETLW   0x32
            RETLW   0x48
            RETLW   0xD3
            RETLW   0x31
            RETLW   0x08
            RETLW   0x8C
            RETLW   0x8F
            RETLW   0x95
            RETLW   0xBD
            RETLW   0xD0
            RETLW   0xE4
            RETLW   0x6D
            RETLW   0x50
            RETLW   0x81
            RETLW   0x20
            RETLW   0x30
            RETLW   0xBB
            RETLW   0x75
            RETLW   0xF5
            RETLW   0xD4
            RETLW   0x7C
            RETLW   0x87
            RETLW   0x2C
            RETLW   0x4E
            RETLW   0xE8
            RETLW   0xF4
            RETLW   0xBE
            RETLW   0x24
            RETLW   0x9E
            RETLW   0x4D
            RETLW   0x80
            RETLW   0x37
            RETLW   0xD2
            RETLW   0x5F
            RETLW   0xDB
            RETLW   0x04
            RETLW   0x7A
            RETLW   0x3F
            RETLW   0x14
            RETLW   0x72
            RETLW   0x67
            RETLW   0x2D
            RETLW   0xCD
            RETLW   0x15
            RETLW   0xA6
            RETLW   0x4C
            RETLW   0x2E
            RETLW   0x3B
            RETLW   0x0C
            RETLW   0x41
            RETLW   0x62
            RETLW   0xFA
            RETLW   0xEE
            RETLW   0x83
            RETLW   0x1E
            RETLW   0xA2
            RETLW   0x01
            RETLW   0x0E
            RETLW   0x7F
            RETLW   0x59
            RETLW   0xC9
            RETLW   0xB9
            RETLW   0xC4
            RETLW   0x9D
            RETLW   0x9B
            RETLW   0x1B
            RETLW   0x9C
            RETLW   0xCA
            RETLW   0xAF
            RETLW   0x3C
            RETLW   0x73
            RETLW   0x1A
            RETLW   0x65
            RETLW   0xB1
            RETLW   0x76
            RETLW   0x84
            RETLW   0x39
            RETLW   0x98
            RETLW   0xE9
            RETLW   0x53
            RETLW   0x94
            RETLW   0xBA
            RETLW   0x1D
            RETLW   0x29
            RETLW   0xCF
            RETLW   0xB4
            RETLW   0x0D
            RETLW   0x05
            RETLW   0x7D
            RETLW   0xD1
            RETLW   0xD7
            RETLW   0x0A
            RETLW   0xA0
            RETLW   0x5C
            RETLW   0x91
            RETLW   0x71
            RETLW   0x92
            RETLW   0x88
            RETLW   0xAB
            RETLW   0x93
            RETLW   0x11
            RETLW   0x8A
            RETLW   0xD6
            RETLW   0x5A
            RETLW   0x77
            RETLW   0xB5
            RETLW   0xC3
            RETLW   0x19
            RETLW   0xC1
            RETLW   0xC7
            RETLW   0x8E
            RETLW   0xF9
            RETLW   0xEC
            RETLW   0x35
            RETLW   0x4B
            RETLW   0xCC
            RETLW   0xD9
            RETLW   0x4A
            RETLW   0x18
            RETLW   0x23
            RETLW   0x9F
            RETLW   0x52
            RETLW   0xDD
            RETLW   0xE3
            RETLW   0xAD
            RETLW   0x7B
            RETLW   0x47
            RETLW   0x97
            RETLW   0x60
            RETLW   0x10
            RETLW   0x43
            RETLW   0xEF
            RETLW   0x07
            RETLW   0xA5
            RETLW   0x49
            RETLW   0xC6
            RETLW   0xB3
            RETLW   0x55
            RETLW   0x28
            RETLW   0x51
            RETLW   0x5D
            RETLW   0x64
            RETLW   0x66
            RETLW   0xFC
            RETLW   0x44
            RETLW   0x42
            RETLW   0xBC
            RETLW   0x26
            RETLW   0x09
            RETLW   0x74
            RETLW   0x6F
            RETLW   0xF7
            RETLW   0x6B
            RETLW   0x4F
            RETLW   0x2F
            RETLW   0xF0
            RETLW   0xEA
            RETLW   0xB8
            RETLW   0xAE
            RETLW   0xF3
            RETLW   0x63
            RETLW   0x6A
            RETLW   0x56
            RETLW   0xB2
            RETLW   0x02
            RETLW   0xD8
            RETLW   0x34
            RETLW   0xA4
            RETLW   0x00
            RETLW   0xE6
            RETLW   0x58
            RETLW   0xEB
            RETLW   0xA3
            RETLW   0x82
            RETLW   0x85
            RETLW   0x45
            RETLW   0xE0
            RETLW   0x89
            RETLW   0x7E
            RETLW   0xFD
            RETLW   0xF2
            RETLW   0x3A
            RETLW   0x36
            RETLW   0x57
            RETLW   0xFF
            RETLW   0x06
            RETLW   0x69
            RETLW   0x54
            RETLW   0x79
            RETLW   0x9A
            RETLW   0xB6
            RETLW   0x6C
            RETLW   0xDC
            RETLW   0x8B
            RETLW   0xA7
            RETLW   0x1F
            RETLW   0x90
            RETLW   0x03
            RETLW   0x17
            RETLW   0x1C
            RETLW   0xED
            RETLW   0xD5
            RETLW   0xAA
            RETLW   0x5E
            RETLW   0xFE
            RETLW   0xDA
            RETLW   0x78
            RETLW   0xB0
            RETLW   0xBF
            RETLW   0x12
            RETLW   0xA8
            RETLW   0x22
            RETLW   0x21
            RETLW   0x3D
            RETLW   0xC2
            RETLW   0xC0
            RETLW   0xB7
            RETLW   0xA9
            RETLW   0xE7
            RETLW   0x33
            RETLW   0xFB
            RETLW   0xF1
            RETLW   0x70
            RETLW   0xE5
            RETLW   0x17
            RETLW   0x96
            RETLW   0xF8
            RETLW   0x8D
            RETLW   0x46
            RETLW   0xA1
            RETLW   0x86
            RETLW   0xE2
            RETLW   0x40
            RETLW   0x38
            RETLW   0xF6
            RETLW   0x68
            RETLW   0x25
            RETLW   0x16
            RETLW   0xAC
            RETLW   0x61
            RETLW   0x27
            RETLW   0xCB
            RETLW   0x5B
            RETLW   0xC8
            RETLW   0x2B
            RETLW   0x0F
            RETLW   0x99
            RETLW   0xDE
            RETLW   0xCE
            RETLW   0xC5

;*************************************************
; function : 30 C1 CLASS instruction             *
; PIN CONTROLLED OPERATIONS                      *
;*************************************************
Instr_30    MOVLW   0x02
            CALL    c14Dc              ; doczytaj -2
            CALL    Receive            ;\ 
            IORWF   0x0F,f
            ANDLW   0x0F
            MOVWF   0x0C               ;/
            CALL    Receive            ;\
            MOVWF   0x0D
            IORWF   0x0F,f
            MOVF    0x0E,f             ;/
            BTFSS   STATUS,Z           ; Skip if zero
            GOTO    TX9000
            INCF    PCLATH,f
            MOVLW   0x90
            MOVF    0x0F,f
            BTFSC   STATUS,Z           ; Skip if not zero
            GOTO    c325c
            CALL    c1ADc
            MOVWF   0x0F
            BTFSS   0x0F,06
            GOTO    c31Ec
            BTFSC   0x0F,03
            GOTO    Zapisz_klucz
            XORLW   0x40
            CALL    c1ADc
            IORWF   0x0C,f
            SWAPF   0x0C,f
            MOVF    0x0F,W
            XORLW   0x40
            INCF    0x0F,f
c31Ec       CALL    i2c_wr
            MOVF    0x0D,W
            CALL    wystaw_byte
            SWAPF   0x0C,W
            CALL    wr_last
            MOVLW   0x40
            XORWF   0x0F,W
c325c       MOVWF   0x0F
            INCF    0x0F,f
            MOVLW   0x90
            CALL    i2c_wr
            MOVF    0x0F,W
            CALL    wr_last
            GOTO    TX9000

;-----------------------------------------------
Zapisz_klucz  
            MOVLW   0x38               ;\ustaw wskaznik bufora
            MOVWF   FSR                ;/
            MOVLW   0x91
            CALL    c125c
            MOVF    0x0C,W
            IORWF   0x3F,f
            CLRF    PCLATH
            MOVF    0x0D,W
            ANDLW   0x07
            ADDWF   PCLATH,f
            MOVF    0x0D,W
            ANDLW   0xF8
            CALL    i2c_wr
            MOVLW   0x08              ; dlugosc
            MOVWF   0x0F              ; licznik
            CALL    wr_i2c_string
            GOTO    TX9000

;**************************************************
; function :  C1 40 CLASS instruction             *
;ENTITLEMENT MANAGEMENT                           *
;**************************************************
Instr_40    MOVF    0x0D,W
            MOVWF   0x17
            BSF     PCLATH,03
            MOVLW   0x34
            SUBWF   0x0F,W
            BTFSS   STATUS,C            ; Skip if carry set
            GOTO    Instr_34
            CALL    Get_CW              ; pobierz dane
            BSF     PCLATH,03
            MOVF    0x17,W
            ANDLW   0x0F
            BTFSC   STATUS,Z            ; Skip if not zero
            GOTO    TX9000              ; zakoncz instrukcje

            MOVLW   0x13
ins_40_loop SUBWF   0x14,f
            MOVF    0x14,W
            MOVWF   FSR
            MOVLW   0x18
            SUBWF   FSR,W
            BTFSS   STATUS,C            ; Skip if carry set
            GOTO    Comp_data
            MOVF    INDF,W
            XORLW   0x90                ; primary key
            BTFSS   STATUS,Z            ; Skip if zero
            GOTO    TX9000              ; zakoncz instrukcje

            MOVF    FSR,W
            SUBLW   0x0A
            MOVWF   EEADR
            MOVLW   0x0C
            CALL    xor_eedat
            CALL    xor_24_30
            CALL    xor_2C_38
            CALL    Encrypt
            CALL    xor_F8_30
            CALL    Comp_key           ; zapisz klucz jesli nowy
            MOVLW   0x0A
            GOTO    ins_40_loop
;--------------------------------------------
Comp_data   ADDLW   0x1F
            MOVWF   FSR
            MOVF    INDF,W
            XORLW   0x21              ; subscription
            BTFSS   STATUS,Z          ; Skip if zero
            GOTO    TX9000            ; zakoncz instrukcje

            CLRF    PCLATH
            MOVLW   0x0E
            INCF    0x17,f
Comp_data_loop  
            ADDLW   0x18
            BTFSC   STATUS,C          ; Skip if no carry
            INCF    PCLATH,f
            DECFSZ  0x17,f
            GOTO    Comp_data_loop
            MOVWF   0x0C
            CALL    i2c_wr            ; adres data ins_12
            INCF    FSR,f
            CALL    i2c_rd            ;\porownaj 1-y byte
            XORWF   INDF,W            ;/
            BTFSS   STATUS,Z          ; Skip if zero
            GOTO    Wr_data           ; zapisz nowa date uprawnien
            INCF    FSR,f
            CALL    i2c_rd            ;\porownaj 2-gi byte
            XORWF   INDF,W            ;/
            BTFSC   STATUS,Z          ; Skip if not zero
            GOTO    TX9000            ; zakoncz instrukcje

            DECF    FSR,f
Wr_data     MOVF    0x0C,W
            CALL    i2c_wr            ; adres
            MOVF    INDF,W
            CALL    wystaw_byte       ;\
            INCF    FSR,f             ; zapisz nowa date
            MOVF    INDF,W
            CALL    wr_last           ;/
            GOTO    TX9000
;---------------------------------------------
; zapisz klucz jesli nowy
Comp_key    INCF    0x14,W
            MOVWF   FSR
            MOVF    INDF,W
            MOVWF   0x0E
            MOVF    0x17,W
            MOVWF   0x0D
            CALL    c112c               ; wylicz adres klucza w EEPROM
            MOVF    0x10,W
            CALL    i2c_wr              ; adres
            MOVLW   0x38                ; adres bufora
            MOVWF   FSR
Comp_key_loop  
            CALL    i2c_rd
            XORWF   INDF,W
            BTFSS   STATUS,Z            ; Skip if zero
            GOTO    Wr_key
            INCF    FSR,f
            BTFSC   FSR,03
            GOTO    Comp_key_loop
            RETURN
;-----------------------------------------------
Wr_key      MOVF    0x10,W
            CALL    i2c_wr              ; adres
            MOVLW   0x08                ;\ licznik
            MOVWF   0x0F                ;/
            CALL    wr_i2c_string
            RETURN
;---------------------------------------
xor_24_30   MOVLW   0x24
            MOVWF   EEADR
            MOVLW   0x30
            CALL    xor_eedat
            RETURN
;---------------------------------------
xor_2C_38  MOVLW   0x2C
            MOVWF   EEADR
            MOVLW   0x38
            CALL    xor_eedat
            RETURN
;-------------------------------------------
; koncowka komendy C1 32
c3A6c       MOVLW   0x83
            CALL    Send
            MOVLW   0x0A
            SUBWF   0x0F,W
            MOVWF   0x10                 ; licznik
            MOVLW   0x08
            MOVWF   0x0F
            BCF     STATUS,C             ; Clear carry flag
            RLF     0x0D,f
            RLF     0x0D,f
            RLF     0x0D,W
            INCF    PCLATH,f
            CALL    EEPROM_send
            MOVLW   0x04
            CALL    Send
c3a6c_loop  DECF    0x10,f
            BTFSC   0x10,07
            GOTO    TX9000
            MOVLW   0xFF
            CALL    Send
            GOTO    c3a6c_loop

Instr_21    MOVF    0x0D,W
            ADDWF   PCLATH,f
            MOVF    0x0E,W
            CALL    i2c_wr
instr_21_loop  
            CALL    i2c_rd
            CALL    Send
            DECFSZ  0x0F,f
            GOTO    instr_21_loop
            GOTO    TX9000

 ;----------------------------------------------
            ORG     0x2000
            DATA    0x0F
            DATA    0x0F
            DATA    0x0F
            DATA    0x0F

;========================================
; fuse
 
            ORG     0x2007
            DATA    0x19

;========================================
; EEPROM
 
            ORG     0x2100
ATR         DATA    0x00
            DATA    0xD6
            DATA    0xB6
            DATA    0x6C
            DATA    0x0E
            DATA    0x04
            DATA    0x30
            DATA    0x54
            DATA    0x96
            DATA    0x40
            DATA    0x01
            DATA    0x00
            DATA    0x11
            DATA    0xF7
            DATA    0x3B
;========================================
;COMMAND
            DATA    0x0E
            DATA    0x16
            DATA    0x12
            DATA    0x34
            DATA    0x32
            DATA    0x20
            DATA    0x30
            DATA    0x21
            DATA    0x3A
            DATA    0x3C
            DATA    0x40
            DATA    0x1A
            DATA    0x5A
;========================================

            END